@@ -8,6 +8,7 @@
  * Description:
  * - Reworked to match AMF B.02 information model Description:
  * - Refactoring of code into several AMF files
+ * - Serializers/deserializers
  *
  * All rights reserved.
  *
@@ -48,7 +49,8 @@
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <assert.h>
+#include <sys/types.h>
+#include <regex.h>
 
 #include "../include/saAis.h"
 #include "../include/saAmf.h"
@@ -58,31 +60,18 @@
 #include "amf.h"
 #include "totem.h"
 #include "print.h"
-
-typedef enum {
-	AMF_HEAD,
-	AMF_APPLICATION,
-	AMF_CLUSTER,
-	AMF_NODE,
-	AMF_SG,
-	AMF_SU,
-	AMF_COMP,
-	AMF_COMP_ENV_VAR,
-	AMF_COMP_CS_TYPE,
-	AMF_SI,
-	AMF_SI_RANKED_SU,
-	AMF_SI_DEPENDENCY,
-	AMF_CSI,
-	AMF_CSI_ATTRIBUTE,
-	AMF_HEALTHCHECK,
-	AMF_CSI_DEPENDENCIES,
-	AMF_CS_TYPE,
-} amf_parse_t;
+#include "aispoll.h"
+#include "main.h"
+#include "service.h"
 
 #ifndef OPENAIS_CLUSTER_STARTUP_TIMEOUT
 #define OPENAIS_CLUSTER_STARTUP_TIMEOUT 5000
 #endif
 
+struct req_exec_amf_msg {
+	mar_req_header_t header;
+};
+
 static const char *presence_state_text[] = {
 	"UNKNOWN",
 	"UNINSTANTIATED",
@@ -90,8 +79,8 @@ static const char *presence_state_text[] = {
 	"INSTANTIATED",
 	"TERMINATING",
 	"RESTARTING",
-	"INSTANTION_FAILED",
-	"TERMINIATION-FAILED"
+	"INSTANTIATION_FAILED",
+	"TERMINATION_FAILED"
 };
 
 static const char *oper_state_text[] = {
@@ -131,21 +120,21 @@ static const char *assignment_state_text[] = {
 
 static int init_category (struct amf_comp *comp, char *loc)
 {
-	 if (strcmp (loc, "sa_aware") == 0) {
-		 comp->saAmfCompCategory = SA_AMF_COMP_SA_AWARE;
-	 } else if (strcmp (loc, "proxy") == 0) {
-		 comp->saAmfCompCategory = SA_AMF_COMP_PROXY;
-	 } else if (strcmp (loc, "proxied") == 0) {
-		 comp->saAmfCompCategory = SA_AMF_COMP_PROXIED;
-	 } else if (strcmp (loc, "local") == 0) {
-		 comp->saAmfCompCategory = SA_AMF_COMP_LOCAL;
-	 } else {
-		 return -1;
-	 }
- 
-	 return 0;
-}	
- 
+	if (strcmp (loc, "sa_aware") == 0) {
+		comp->saAmfCompCategory = SA_AMF_COMP_SA_AWARE;
+	} else if (strcmp (loc, "proxy") == 0) {
+		comp->saAmfCompCategory = SA_AMF_COMP_PROXY;
+	} else if (strcmp (loc, "proxied") == 0) {
+		comp->saAmfCompCategory = SA_AMF_COMP_PROXIED;
+	} else if (strcmp (loc, "local") == 0) {
+		comp->saAmfCompCategory = SA_AMF_COMP_LOCAL;
+	} else {
+		return -1;
+	}
+
+	return 0;
+}   
+
 static int init_capability (struct amf_comp *comp, char *loc)
 {
 	if (strcmp (loc, "x_active_and_y_standby") == 0) {
@@ -165,10 +154,10 @@ static int init_capability (struct amf_comp *comp, char *loc)
 	} else {
 		return -1;
 	}
-  
+
 	return 0;
 }
- 
+
 static int init_recovery_on_error (struct amf_comp *comp, char *loc)
 {
 	if (strcmp (loc, "component_restart") == 0) {
@@ -185,6 +174,8 @@ static int init_recovery_on_error (struct amf_comp *comp, char *loc)
 		comp->saAmfCompRecoveryOnError = SA_AMF_APPLICATION_RESTART;
 	} else if (strcmp (loc, "cluster_reset") == 0) {
 		comp->saAmfCompRecoveryOnError = SA_AMF_CLUSTER_RESET;
+	} else if (strcmp (loc, "no_recomondation") == 0) {
+		comp->saAmfCompRecoveryOnError = SA_AMF_NO_RECOMMENDATION;
 	} else {
 		return -1;
 	}
@@ -251,7 +242,7 @@ struct amf_cluster *amf_config_read (char **error_string)
 	char *line;
 	FILE *fp;
 	char *filename;
-	amf_parse_t current_parse = AMF_HEAD;
+	amf_object_type_t current_parse = AMF_NONE;
 	int line_number = 0;
 	char *loc;
 	int i;
@@ -276,8 +267,7 @@ struct amf_cluster *amf_config_read (char **error_string)
 	int                       csi_dependencies_cnt = 0;
 	char                     *error_reason = NULL;
 	char                     *value;
-
-	filename = getenv("OPENAIS_AMF_CONFIG_FILE");
+	filename = getenv ("OPENAIS_AMF_CONFIG_FILE");
 	if (!filename) {
 		filename = "/etc/ais/amf.conf";
 	}
@@ -285,12 +275,12 @@ struct amf_cluster *amf_config_read (char **error_string)
 	fp = fopen (filename, "r");
 	if (fp == 0) {
 		sprintf (buf, "Can't read %s file reason = (%s).\n",
-				 filename, strerror (errno));
+			filename, strerror (errno));
 		*error_string = buf;
 		return NULL;
 	}
 
-	cluster = amf_cluster_create ();
+	cluster = amf_cluster_new ();
 	assert (cluster != NULL);
 
 	while (fgets (buf, 255, fp)) {
@@ -320,7 +310,7 @@ struct amf_cluster *amf_config_read (char **error_string)
 		error_reason = line;
 		error_reason = NULL;
 		switch (current_parse) {
-		case AMF_HEAD:
+		case AMF_NONE:
 			if ((loc = strstr_rs (line, "safAmfCluster=")) != 0) {
 				setSaNameT (&cluster->name, trim_str (loc));
 				current_parse = AMF_CLUSTER;
@@ -335,22 +325,11 @@ struct amf_cluster *amf_config_read (char **error_string)
 			} else if ((loc = strstr_rs (line, "saAmfClusterStartupTimeout=")) != 0) {
 				cluster->saAmfClusterStartupTimeout = atol(loc);
 			} else if ((loc = strstr_rs (line, "safAmfNode=")) != 0) {
-				node = calloc (1, sizeof (struct amf_node));
-				node->next = cluster->node_head;
+				node = amf_node_new (cluster, trim_str (loc));
 				cluster->node_head = node;
-				node->saAmfNodeAdminState = SA_AMF_ADMIN_UNLOCKED;
-				node->saAmfNodeAutoRepair = SA_TRUE;
-				node->cluster = cluster;
-				node->saAmfNodeSuFailOverProb = -1;
-				node->saAmfNodeSuFailoverMax = ~0;
-				setSaNameT (&node->name, trim_str (loc));
 				current_parse = AMF_NODE;
 			} else if ((loc = strstr_rs (line, "safApp=")) != 0) {
-				app = calloc (1, sizeof (struct amf_application));
-				app->next = cluster->application_head;
-				cluster->application_head = app;
-				app->cluster = cluster;
-				app->saAmfApplicationAdminState = SA_AMF_ADMIN_UNLOCKED;
+				app = amf_application_new (cluster);
 				setSaNameT (&app->name, trim_str (loc));
 				current_parse = AMF_APPLICATION;
 				sg_cnt = 0;
@@ -363,17 +342,19 @@ struct amf_cluster *amf_config_read (char **error_string)
 				if (cluster->saAmfClusterStartupTimeout == 0) {
 					cluster->saAmfClusterStartupTimeout = OPENAIS_CLUSTER_STARTUP_TIMEOUT;
 				}
-				current_parse = AMF_HEAD;
+				current_parse = AMF_NONE;
 			} else {
 				goto parse_error;
 			}
 			break;
 
 		case AMF_NODE:
-			if ((loc = strstr_rs (line, "saAmfNodeSuFailOverProb")) != 0) {
+			if ((loc = strstr_rs (line, "saAmfNodeSuFailOverProb=")) != 0) {
 				node->saAmfNodeSuFailOverProb = atol(loc);
-			} else if ((loc = strstr_rs (line, "saAmfNodeSuFailoverMax")) != 0) {
+			} else if ((loc = strstr_rs (line, "saAmfNodeSuFailoverMax=")) != 0) {
 				node->saAmfNodeSuFailoverMax = atol(loc);
+			} else if ((loc = strstr_rs (line, "saAmfNodeClmNode=")) != 0) {
+				setSaNameT (&node->saAmfNodeClmNode, trim_str (loc));
 			} else if ((loc = strstr_rs (line, "saAmfNodeAutoRepair=")) != 0) {
 				if (strcmp (loc, "true") == 0) {
 					node->saAmfNodeAutoRepair = SA_TRUE;
@@ -407,6 +388,10 @@ struct amf_cluster *amf_config_read (char **error_string)
 					error_reason = "saAmfNodeSuFailoverMax missing";
 					goto parse_error;
 				}
+				if (node->saAmfNodeClmNode.length == 0) {
+					error_reason = "saAmfNodeClmNode missing";
+					goto parse_error;
+				}
 				current_parse = AMF_CLUSTER;
 			} else {
 				goto parse_error;
@@ -415,35 +400,20 @@ struct amf_cluster *amf_config_read (char **error_string)
 
 		case AMF_APPLICATION:
 			if ((loc = strstr_rs (line, "clccli_path=")) != 0) {
+				app->clccli_path = amf_malloc (strlen (loc) + 1);
 				strcpy (app->clccli_path, loc);
 			} else if ((loc = strstr_rs (line, "safSg=")) != 0) {
+				sg = amf_sg_new (app, trim_str (loc));
 				sg_cnt++;
-				sg = calloc (1, sizeof (struct amf_sg));
-				sg->next = app->sg_head;
-				app->sg_head = sg;
-				sg->saAmfSGAdminState = SA_AMF_ADMIN_UNLOCKED;
-				sg->saAmfSGNumPrefActiveSUs = 1;
-				sg->saAmfSGNumPrefStandbySUs = 1;
-				sg->saAmfSGNumPrefInserviceSUs = ~0;
-				sg->saAmfSGCompRestartProb = -1;
-				sg->saAmfSGCompRestartMax = ~0;
-				sg->saAmfSGSuRestartProb = -1;
-				sg->saAmfSGSuRestartMax = ~0;
-				sg->saAmfSGAutoAdjustProb = -1;
-				sg->saAmfSGAutoRepair = SA_TRUE;
-				sg->application = app;
+				sg->recovery_scope.comp = NULL;
+				sg->recovery_scope.event_type = 0;
+				sg->recovery_scope.node = NULL;
+				sg->recovery_scope.sis = NULL;
+				sg->recovery_scope.sus = NULL;
 				current_parse = AMF_SG;
 				su_cnt = 0;
-				setSaNameT (&sg->name, trim_str (loc));
 			} else if ((loc = strstr_rs (line, "safSi=")) != 0) {
-				si = calloc (1, sizeof (struct amf_si));
-				si->next = app->si_head;
-				app->si_head = si;
-				si->application = app;
-				si->saAmfSIPrefActiveAssignments = 1;
-				si->saAmfSIPrefStandbyAssignments = 1;
-				setSaNameT (&si->name, trim_str (loc));
-				si->saAmfSIAdminState = SA_AMF_ADMIN_UNLOCKED;
+				si = amf_si_new (app, trim_str (loc));
 				current_parse = AMF_SI;
 			} else if ((loc = strstr_rs (line, "safCSType=")) != 0) {
 				current_parse = AMF_CS_TYPE;
@@ -451,7 +421,7 @@ struct amf_cluster *amf_config_read (char **error_string)
 				if (sg_cnt == 1) {
 					for (si = app->si_head; si != NULL; si = si->next) {
 						memcpy (&si->saAmfSIProtectedbySG, &sg->name,
-								sizeof (SaNameT));
+							sizeof (SaNameT));
 					}
 				} else {
 					for (si = app->si_head; si != NULL; si = si->next) {
@@ -470,6 +440,7 @@ struct amf_cluster *amf_config_read (char **error_string)
 
 		case AMF_SG:
 			if ((loc = strstr_rs (line, "clccli_path=")) != 0) {
+				sg->clccli_path = amf_malloc (strlen (loc) + 1);
 				strcpy (sg->clccli_path, loc);
 			} else if ((loc = strstr_rs (line, "saAmfSGRedundancyModel=")) != 0) {
 				if (strcmp (loc, "2n") == 0) {
@@ -491,6 +462,10 @@ struct amf_cluster *amf_config_read (char **error_string)
 				sg->saAmfSGNumPrefActiveSUs = atoi (loc);
 			} else if ((loc = strstr_rs (line, "saAmfSGNumPrefStandbySUs=")) != 0) {
 				sg->saAmfSGNumPrefStandbySUs = atoi (loc);
+			} else if ((loc = strstr_rs (line, "saAmfSGNumPrefInserviceSUs=")) != 0) {
+				sg->saAmfSGNumPrefInserviceSUs = atoi (loc);
+			} else if ((loc = strstr_rs (line, "saAmfSGNumPrefAssignedSUs=")) != 0) {
+				sg->saAmfSGNumPrefAssignedSUs = atoi (loc);
 			} else if ((loc = strstr_rs (line, "saAmfSGMaxActiveSIsperSUs=")) != 0) {
 				sg->saAmfSGMaxActiveSIsperSUs = atoi (loc);
 			} else if ((loc = strstr_rs (line, "saAmfSGMaxStandbySIsperSUs=")) != 0) {
@@ -508,17 +483,8 @@ struct amf_cluster *amf_config_read (char **error_string)
 			} else if ((loc = strstr_rs (line, "saAmfSGAutoRepair=")) != 0) {
 				sg->saAmfSGAutoRepair = atoi (loc);
 			} else if ((loc = strstr_rs (line, "safSu=")) != 0) {
-				su = calloc (1, sizeof (struct amf_su));
+				su = amf_su_new (sg, trim_str (loc));
 				su_cnt++;
-				su->next = sg->su_head;
-				sg->su_head = su;
-				su->sg = sg;
-				su->saAmfSUAdminState = SA_AMF_ADMIN_UNLOCKED;
-				su->saAmfSUOperState = SA_AMF_OPERATIONAL_DISABLED;
-				su->saAmfSUPresenceState = SA_AMF_PRESENCE_UNINSTANTIATED;
-				su->restart_control_state = SU_RC_ESCALATION_LEVEL_0;
-				su->saAmfSUFailover = 1;
-				setSaNameT (&su->name, trim_str (loc));
 				current_parse = AMF_SU;
 			} else if (strstr_rs (line, "}")) {
 				if (sg->saAmfSGRedundancyModel == 0) {
@@ -552,6 +518,10 @@ struct amf_cluster *amf_config_read (char **error_string)
 				if (sg->saAmfSGNumPrefInserviceSUs == ~0) {
 					sg->saAmfSGNumPrefInserviceSUs = su_cnt;
 				}
+				if (sg->saAmfSGNumPrefAssignedSUs == ~0) {
+					sg->saAmfSGNumPrefAssignedSUs =
+						sg->saAmfSGNumPrefInserviceSUs;
+				}
 				current_parse = AMF_APPLICATION;
 			} else {
 				goto parse_error;
@@ -566,14 +536,14 @@ struct amf_cluster *amf_config_read (char **error_string)
 			} else if ((loc = strstr_rs (line, "saAmfSUFailover=")) != 0) {
 				su->saAmfSUFailover = atoi (loc);
 			} else if ((loc = strstr_rs (line, "clccli_path=")) != 0) {
+				su->clccli_path = amf_malloc (strlen (loc) + 1);
 				strcpy (su->clccli_path, loc);
 			} else if ((loc = strstr_rs (line, "saAmfSUHostedByNode=")) != 0) {
 				setSaNameT (&su->saAmfSUHostedByNode, loc);
 			} else if ((loc = strstr_rs (line, "safComp=")) != 0) {
-				comp = amf_comp_create (su);
+				comp = amf_comp_new (su, trim_str (loc));
 				comp_env_var_cnt = 0;
 				comp_cs_type_cnt = 0;
-				setSaNameT (&comp->name, trim_str (loc));
 				current_parse = AMF_COMP;
 			} else if (strstr_rs (line, "}")) {
 				if (su->saAmfSUNumComponents == 0) {
@@ -600,6 +570,7 @@ struct amf_cluster *amf_config_read (char **error_string)
 
 		case AMF_COMP:
 			if ((loc = strstr_rs (line, "clccli_path=")) != 0) {
+				comp->clccli_path = amf_malloc (strlen (loc) + 1);
 				strcpy (comp->clccli_path, loc);
 			} else if ((loc = strstr_rs (line, "saAmfCompCsTypes{")) != 0) {
 				current_parse = AMF_COMP_CS_TYPE;
@@ -624,10 +595,10 @@ struct amf_cluster *amf_config_read (char **error_string)
 			} else if ((loc = strstr_rs(line, "saAmfCompDefaultCallbackTimeOut=")) != 0) {
 				comp->saAmfCompDefaultCallbackTimeOut = atol (loc);
 			} else if ((loc = strstr_rs (line, "saAmfCompInstantiateCmdArgv=")) != 0) {
-				comp->saAmfCompInstantiateCmdArgv = malloc (strlen(loc) + 1);
+				comp->saAmfCompInstantiateCmdArgv = amf_malloc (strlen(loc) + 1);
 				strcpy (comp->saAmfCompInstantiateCmdArgv, loc);
 			} else if ((loc = strstr_rs ( line, "saAmfCompInstantiateCmd=")) != 0) {
-				comp->saAmfCompInstantiateCmd = malloc (strlen(loc) + 1);
+				comp->saAmfCompInstantiateCmd = amf_malloc (strlen(loc) + 1);
 				strcpy (comp->saAmfCompInstantiateCmd, loc);
 			} else if ((loc = strstr_rs(line, "saAmfCompInstantiateTimeout=")) != 0) {
 				comp->saAmfCompInstantiateTimeout = atol (loc);
@@ -640,18 +611,18 @@ struct amf_cluster *amf_config_read (char **error_string)
 			} else if ((loc = strstr_rs(line, "saAmfCompDelayBetweenInstantiateAttempts=")) != 0) {
 				comp->saAmfCompDelayBetweenInstantiateAttempts = atol (loc);
 			} else if ((loc = strstr_rs (line, "saAmfCompTerminateCmdArgv=")) != 0) {
-				comp->saAmfCompTerminateCmdArgv = malloc (strlen(loc) + 1);
+				comp->saAmfCompTerminateCmdArgv = amf_malloc (strlen(loc) + 1);
 				strcpy (comp->saAmfCompTerminateCmdArgv, loc);
 			} else if ((loc = strstr_rs (line, "saAmfCompTerminateCmd=")) != 0) {
-				comp->saAmfCompTerminateCmd = malloc (strlen(loc) + 1);
+				comp->saAmfCompTerminateCmd = amf_malloc (strlen(loc) + 1);
 				strcpy (comp->saAmfCompTerminateCmd, loc);
 			} else if ((loc = strstr_rs(line, "saAmfCompTerminateTimeout=")) != 0) {
 				comp->saAmfCompTerminateTimeout = atol (loc);
 			} else if ((loc = strstr_rs (line, "saAmfCompCleanupCmdArgv=")) != 0) {
-				comp->saAmfCompCleanupCmdArgv = malloc (strlen(loc) + 1);
+				comp->saAmfCompCleanupCmdArgv = amf_malloc (strlen(loc) + 1);
 				strcpy (comp->saAmfCompCleanupCmdArgv, loc);
 			} else if ((loc = strstr_rs (line, "saAmfCompCleanupCmd=")) != 0) {
-				comp->saAmfCompCleanupCmd = malloc (strlen(loc) + 1);
+				comp->saAmfCompCleanupCmd = amf_malloc (strlen(loc) + 1);
 				strcpy (comp->saAmfCompCleanupCmd, loc);
 			} else if ((loc = strstr_rs(line, "saAmfCompCleanupTimeout=")) != 0) {
 				comp->saAmfCompCleanupTimeout = atol (loc);
@@ -668,7 +639,7 @@ struct amf_cluster *amf_config_read (char **error_string)
 					error_reason = "bad value";
 					goto parse_error;
 				}
-			} else if ((loc = strstr_rs (line, "saAmfCompDisableRestart")) != 0) {
+			} else if ((loc = strstr_rs (line, "saAmfCompDisableRestart=")) != 0) {
 				if (strcmp (loc, "false") == 0) {
 					comp->saAmfCompDisableRestart = SA_FALSE;
 				} else if (strcmp (loc, "true") == 0) {
@@ -741,9 +712,9 @@ struct amf_cluster *amf_config_read (char **error_string)
 			} else {
 				comp_cs_type_cnt++;
 				comp->saAmfCompCsTypes = realloc (comp->saAmfCompCsTypes,
-												 (comp_cs_type_cnt + 1) * sizeof(SaNameT));
+					(comp_cs_type_cnt + 1) * sizeof(SaNameT));
 				comp->saAmfCompCsTypes[comp_cs_type_cnt] = NULL;
-				comp->saAmfCompCsTypes[comp_cs_type_cnt - 1] = malloc (sizeof(SaNameT));
+				comp->saAmfCompCsTypes[comp_cs_type_cnt - 1] = amf_malloc (sizeof(SaNameT));
 				setSaNameT (comp->saAmfCompCsTypes[comp_cs_type_cnt - 1], line);
 			}
 			break;
@@ -754,9 +725,9 @@ struct amf_cluster *amf_config_read (char **error_string)
 			} else if ((loc = strchr (line, '=')) != 0) {
 				comp_env_var_cnt++;
 				comp->saAmfCompCmdEnv = realloc (comp->saAmfCompCmdEnv,
-												 (comp_env_var_cnt + 1) * sizeof(SaStringT));
+					(comp_env_var_cnt + 1) * sizeof(SaStringT));
 				comp->saAmfCompCmdEnv[comp_env_var_cnt] = NULL;
-				env_var = comp->saAmfCompCmdEnv[comp_env_var_cnt - 1] = malloc (strlen (line) + 1);
+				env_var = comp->saAmfCompCmdEnv[comp_env_var_cnt - 1] = amf_malloc (strlen (line) + 1);
 				strcpy (env_var, line);
 			} else {
 				goto parse_error;
@@ -796,9 +767,9 @@ struct amf_cluster *amf_config_read (char **error_string)
 				csi->si = si;
 				setSaNameT (&csi->name, trim_str (loc));
 				current_parse = AMF_CSI;
-			} else if ((loc = strstr_rs (line, "saAmfSIProtectedbySG{")) != 0) {
+			} else if ((loc = strstr_rs (line, "saAmfSIProtectedbySG=")) != 0) {
 				setSaNameT (&si->saAmfSIProtectedbySG, loc);
-			} else if ((loc = strstr_rs (line, "saAmfSIRank{")) != 0) {
+			} else if ((loc = strstr_rs (line, "saAmfSIRank=")) != 0) {
 				si->saAmfSIRank = atoi (loc);
 			} else if ((loc = strstr_rs (line, "saAmfSINumCSIs=")) != 0) {
 				si->saAmfSINumCSIs = atoi (loc);
@@ -844,7 +815,7 @@ struct amf_cluster *amf_config_read (char **error_string)
 				attribute = calloc (1, sizeof(struct amf_csi_attribute));
 				attribute->next = csi->attributes_head;
 				csi->attributes_head = attribute;
-				attribute->name = malloc (strlen (loc) + 1);
+				attribute->name = amf_malloc (strlen (loc) + 1);
 				strcpy (attribute->name, trim_str (loc));
 				csi_attr_cnt = 1;
 				current_parse = AMF_CSI_ATTRIBUTE;
@@ -871,7 +842,7 @@ struct amf_cluster *amf_config_read (char **error_string)
 					(csi_dependencies_cnt + 1) * sizeof(SaNameT));
 				csi->saAmfCSIDependencies[csi_dependencies_cnt] = NULL;
 				csi->saAmfCSIDependencies[csi_dependencies_cnt - 1] =
-					malloc (sizeof(SaNameT));
+					amf_malloc (sizeof(SaNameT));
 				setSaNameT (
 					csi->saAmfCSIDependencies[csi_dependencies_cnt - 1], loc);
 			} else {
@@ -887,7 +858,7 @@ struct amf_cluster *amf_config_read (char **error_string)
 				attribute->value = realloc (attribute->value,
 					sizeof (SaStringT) * (csi_attr_cnt + 1));
 				attribute->value[csi_attr_cnt - 1] =
-					malloc (strlen (value) + 1);
+					amf_malloc (strlen (value) + 1);
 				strcpy (attribute->value[csi_attr_cnt - 1], value);
 				attribute->value[csi_attr_cnt] = NULL;
 				csi_attr_cnt++;
@@ -908,11 +879,15 @@ struct amf_cluster *amf_config_read (char **error_string)
 	}
 
 	fclose (fp);
+
+	sprintf (buf, "Successfully read AMF configuration file '%s'.\n", filename);
+	*error_string = buf;
+
 	return cluster;
 
 parse_error:
 	sprintf (buf, "parse error at %s: %d: %s\n",
-			 filename, line_number, error_reason);
+		filename, line_number, error_reason);
 	*error_string = buf;
 	fclose (fp);
 	return NULL;
@@ -931,7 +906,7 @@ static void print_si_assignment (struct amf_su *su,
 {
 	log_printf (LOG_INFO, "          safSi=%s\n", si_assignment->si->name.value);
 	log_printf (LOG_INFO, "            HA state: %s\n",
-			ha_state_text[si_assignment->saAmfSISUHAState]);
+		ha_state_text[si_assignment->saAmfSISUHAState]);
 }
 
 void amf_runtime_attributes_print (struct amf_cluster *cluster)
@@ -948,49 +923,55 @@ void amf_runtime_attributes_print (struct amf_cluster *cluster)
 	log_printf (LOG_INFO, "===================================================");
 	log_printf (LOG_INFO, "safCluster=%s", getSaNameT(&cluster->name));
 	log_printf (LOG_INFO, "  admin state: %s\n",
-			admin_state_text[cluster->saAmfClusterAdminState]);
+		admin_state_text[cluster->saAmfClusterAdminState]);
+	log_printf (LOG_INFO, "  state:       %u\n", cluster->acsm_state);
 	for (node = cluster->node_head; node != NULL; node = node->next) {
 		log_printf (LOG_INFO, "  safNode=%s\n", getSaNameT (&node->name));
+		log_printf (LOG_INFO, "    CLM Node:    %s\n", getSaNameT (&node->saAmfNodeClmNode));
+		log_printf (LOG_INFO, "    node ID:     %u\n", node->nodeid);
 		log_printf (LOG_INFO, "    admin state: %s\n",
-				admin_state_text[node->saAmfNodeAdminState]);
+			admin_state_text[node->saAmfNodeAdminState]);
 		log_printf (LOG_INFO, "    oper state:  %s\n",
-				oper_state_text[node->saAmfNodeOperState]);
+			oper_state_text[node->saAmfNodeOperState]);
+		log_printf (LOG_INFO, "    acsm state:  %u\n", node->acsm_state);
 	}
 	for (app = cluster->application_head; app != NULL; app = app->next) {
 		log_printf (LOG_INFO, "  safApp=%s\n", getSaNameT(&app->name));
 		log_printf (LOG_INFO, "    admin state: %s\n",
-				admin_state_text[app->saAmfApplicationAdminState]);
+			admin_state_text[app->saAmfApplicationAdminState]);
 		log_printf (LOG_INFO, "    num_sg:      %d\n", app->saAmfApplicationCurrNumSG);
-		for (sg = app->sg_head;	sg != NULL; sg = sg->next) {
-			log_printf (LOG_INFO, "    safSG=%s\n", getSaNameT(&sg->name));
+		for (sg = app->sg_head; sg != NULL; sg = sg->next) {
+			log_printf (LOG_INFO, "    safSg=%s\n", getSaNameT(&sg->name));
+			log_printf (LOG_INFO, "      avail_state:        %u\n",
+				sg->avail_state);
 			log_printf (LOG_INFO, "      admin state:        %s\n",
-					admin_state_text[sg->saAmfSGAdminState]);
+				admin_state_text[sg->saAmfSGAdminState]);
 			log_printf (LOG_INFO, "      assigned SUs        %d\n",
-					sg->saAmfSGNumCurrAssignedSUs);
+				sg->saAmfSGNumCurrAssignedSUs);
 			log_printf (LOG_INFO, "      non inst. spare SUs %d\n",
-					sg->saAmfSGNumCurrNonInstantiatedSpareSUs);
+				sg->saAmfSGNumCurrNonInstantiatedSpareSUs);
 			log_printf (LOG_INFO, "      inst. spare SUs     %d\n",
-					sg->saAmfSGNumCurrInstantiatedSpareSUs);
+				sg->saAmfSGNumCurrInstantiatedSpareSUs);
 			for (su = sg->su_head; su != NULL; su = su->next) {
 				log_printf (LOG_INFO, "      safSU=%s\n", getSaNameT(&su->name));
 				log_printf (LOG_INFO, "        oper state:      %s\n",
-						oper_state_text[su->saAmfSUOperState]);
+					oper_state_text[su->saAmfSUOperState]);
 				log_printf (LOG_INFO, "        admin state:     %s\n",
-						admin_state_text[su->saAmfSUAdminState]);
+					admin_state_text[su->saAmfSUAdminState]);
 				log_printf (LOG_INFO, "        readiness state: %s\n",
 					readiness_state_text[amf_su_get_saAmfSUReadinessState (su)]);
 				log_printf (LOG_INFO, "        presence state:  %s\n",
-						presence_state_text[su->saAmfSUPresenceState]);
+					presence_state_text[su->saAmfSUPresenceState]);
 				log_printf (LOG_INFO, "        hosted by node   %s\n",
-						su->saAmfSUHostedByNode.value);
+					su->saAmfSUHostedByNode.value);
 				log_printf (LOG_INFO, "        num active SIs   %d\n",
-						amf_su_get_saAmfSUNumCurrActiveSIs (su));
+					amf_su_get_saAmfSUNumCurrActiveSIs (su));
 				log_printf (LOG_INFO, "        num standby SIs  %d\n",
-						amf_su_get_saAmfSUNumCurrStandbySIs (su));
+					amf_su_get_saAmfSUNumCurrStandbySIs (su));
 				log_printf (LOG_INFO, "        restart count    %d\n",
-						su->saAmfSURestartCount);
+					su->saAmfSURestartCount);
 				log_printf (LOG_INFO, "        restart control state %d\n",
-							su->restart_control_state);
+					su->restart_control_state);
 				log_printf (LOG_INFO, "        SU failover cnt  %d\n", su->su_failover_cnt);
 				log_printf (LOG_INFO, "        assigned SIs:");
 				amf_su_foreach_si_assignment (su, print_si_assignment);
@@ -1008,7 +989,7 @@ void amf_runtime_attributes_print (struct amf_cluster *cluster)
 					log_printf (LOG_INFO, "          num standby CSIs %d\n",
 						amf_comp_get_saAmfCompNumCurrStandbyCsi (comp));
 					log_printf (LOG_INFO, "          restart count    %d\n",
-							comp->saAmfCompRestartCount);
+						comp->saAmfCompRestartCount);
 					log_printf (LOG_INFO, "          assigned CSIs:");
 					amf_comp_foreach_csi_assignment (
 						comp, print_csi_assignment);
@@ -1021,11 +1002,11 @@ void amf_runtime_attributes_print (struct amf_cluster *cluster)
 				admin_state_text[si->saAmfSIAdminState]);
 			log_printf (LOG_INFO, "      assignm. state:      %s\n",
 				assignment_state_text[
-					amf_si_get_saAmfSIAssignmentState (si)]);
+				amf_si_get_saAmfSIAssignmentState (si)]);
 			log_printf (LOG_INFO, "      active assignments:  %d\n",
-					amf_si_get_saAmfSINumCurrActiveAssignments (si));
+				amf_si_get_saAmfSINumCurrActiveAssignments (si));
 			log_printf (LOG_INFO, "      standby assignments: %d\n",
-					amf_si_get_saAmfSINumCurrStandbyAssignments (si));
+				amf_si_get_saAmfSINumCurrStandbyAssignments (si));
 			for (csi = si->csi_head; csi != NULL; csi = csi->next) {
 				log_printf (LOG_INFO, "      safCsi=%s\n", getSaNameT(&csi->name));
 			}
@@ -1043,22 +1024,22 @@ int amf_enabled (struct objdb_iface_ver0 *objdb)
 
 	objdb->object_find_reset (OBJECT_PARENT_HANDLE);
 	if (objdb->object_find (
-			OBJECT_PARENT_HANDLE,
-			"amf",
-			strlen ("amf"),
-			&object_service_handle) == 0) {
+		OBJECT_PARENT_HANDLE,
+		"amf",
+		strlen ("amf"),
+		&object_service_handle) == 0) {
 
 		value = NULL;
-		if ( !objdb->object_key_get (object_service_handle,
-							"mode",
-							strlen ("mode"),
-							(void *)&value,
-							NULL) && value) {
+		if (!objdb->object_key_get (object_service_handle,
+			"mode",
+			strlen ("mode"),
+			(void *)&value,
+			NULL) && value) {
 
 			if (strcmp (value, "enabled") == 0) {
 				enabled = 1;
 			} else
-			if (strcmp (value, "disabled") == 0) {
+				if (strcmp (value, "disabled") == 0) {
 				enabled = 0;
 			}
 		}
@@ -1097,3 +1078,368 @@ const char *amf_assignment_state (int state)
 	return assignment_state_text[state];
 }
 
+#define ALIGN_ADDR(addr) ((addr) + (4 - ((unsigned long)(addr) % 4)))
+
+char *amf_serialize_SaNameT (char *buf, int *size, int *offset, SaNameT *name)
+{
+	char *tmp = buf;
+
+	if ((*size - *offset ) < sizeof (SaNameT)) {
+		*size += sizeof (SaNameT);
+		tmp = realloc (buf, *size);
+		if (tmp == NULL) {
+			openais_exit_error (AIS_DONE_OUT_OF_MEMORY);
+		}
+	}
+
+	memcpy (&tmp[*offset], name, sizeof (SaNameT));
+
+	(*offset) += sizeof (SaNameT);
+
+	return tmp;
+}
+
+char *amf_serialize_SaStringT (char *buf, int *size, int *offset, SaStringT str)
+{
+	unsigned int len;
+
+	if (str != NULL) {
+		len = strlen ((char*)str);
+	} else {
+		len = 0;
+	}
+
+	return amf_serialize_opaque (buf, size, offset, str, len);
+}
+
+char *amf_serialize_SaUint16T (char *buf, int *size, int *offset, SaUint16T num)
+{
+	char *tmp = buf;
+
+	if ((*size - *offset ) < sizeof (SaUint16T)) {
+		*size += sizeof (SaUint16T);
+		tmp = realloc (buf, *size);
+		if (tmp == NULL) {
+			openais_exit_error (AIS_DONE_OUT_OF_MEMORY);
+		}
+	}
+
+	*((SaUint16T *)&tmp[*offset]) = num;
+	(*offset) += sizeof (SaUint16T);
+
+	return tmp;
+}
+
+char *amf_serialize_SaUint32T (char *buf, int *size, int *offset, SaUint32T num)
+{
+	char *tmp = buf;
+
+	if ((*size - *offset ) < sizeof (SaUint32T)) {
+		*size += sizeof (SaUint32T);
+		tmp = realloc (buf, *size);
+		if (tmp == NULL) {
+			openais_exit_error (AIS_DONE_OUT_OF_MEMORY);
+		}
+	}
+
+	*((SaUint32T *)&tmp[*offset]) = num;
+	(*offset) += sizeof (SaUint32T);
+
+	return tmp;
+}
+
+char *amf_serialize_opaque (
+	char *buf, int *size, int *offset, void *src, int cnt)
+{
+	unsigned int required_size;
+	char *tmp = buf;
+
+	required_size = cnt + sizeof (SaUint32T);
+
+	if ((*size - *offset ) < required_size) {
+		*size += required_size;
+		tmp = realloc (buf, *size);
+		if (tmp == NULL) {
+			openais_exit_error (AIS_DONE_OUT_OF_MEMORY);
+		}
+	}
+
+	*((SaUint32T *)&tmp[*offset]) = cnt;
+	(*offset) += sizeof (SaUint32T);
+	memcpy (&tmp[*offset], src, cnt);
+	(*offset) += cnt;
+
+	return tmp;
+}
+
+char *amf_deserialize_SaNameT (char *buf, SaNameT *name)
+{
+	memcpy (name, buf, sizeof (SaNameT));
+	return (buf + sizeof (SaNameT));
+}
+
+char *amf_deserialize_SaStringT (char *buf, SaStringT *str)
+{
+	int len;
+	char *tmp, *tmp_str;
+
+	len = *((SaUint32T *)buf);
+	tmp = buf + sizeof (SaUint32T);
+
+	if (len > 0) {
+		tmp_str = amf_malloc (len + 1);
+		memcpy (tmp_str, tmp, len);
+		tmp_str[len] = '\0';
+		*str = tmp_str;
+	} else {
+		*str = NULL;
+	}
+
+	tmp += len;
+
+	return tmp;
+}
+
+char *amf_deserialize_SaUint16T (char *buf, SaUint16T *num)
+{
+	*num = *((SaUint16T *)buf);
+	return buf + sizeof (SaUint16T);
+}
+
+char *amf_deserialize_SaUint32T (char *buf, SaUint32T *num)
+{
+	*num = *((SaUint32T *)buf);
+	return buf + sizeof (SaUint32T);
+}
+
+char *amf_deserialize_opaque (char *buf, void *dst, int *cnt)
+{
+	*cnt = *((SaUint32T *)buf);
+	memcpy (dst, buf + sizeof (SaUint32T), *cnt);
+	return buf + *cnt + sizeof (SaUint32T);
+}
+
+void *_amf_malloc (size_t size, char *file, unsigned int line)
+{
+	void *tmp = malloc (size);
+
+	if (tmp == NULL) {
+		log_printf (LOG_LEVEL_ERROR, "AMF out-of-memory at %s:%u", file, line);
+		openais_exit_error (AIS_DONE_OUT_OF_MEMORY);
+	}
+
+	return tmp;
+}
+
+void *_amf_calloc (size_t nmemb, size_t size, char *file, unsigned int line)
+{
+	void *tmp = calloc (nmemb, size);
+
+	if (tmp == NULL) {
+		log_printf (LOG_LEVEL_ERROR, "AMF out-of-memory at %s:%u", file, line);
+		openais_exit_error (AIS_DONE_OUT_OF_MEMORY);
+	}
+
+	return tmp;
+}
+
+void *_amf_realloc (void* ptr, size_t size, char *file, unsigned int line)
+{
+	void *tmp = realloc (ptr, size);
+
+	if (tmp == NULL) {
+		log_printf (LOG_LEVEL_ERROR, "AMF out-of-memory at %s:%u", file, line);
+		openais_exit_error (AIS_DONE_OUT_OF_MEMORY);
+	}
+
+	return tmp;
+}
+
+int sa_amf_grep_one_sub_match(const char *string, char *pattern, 
+	SaNameT *matches_arr)
+{
+	int             status;
+	regex_t         re;
+	size_t          nmatch = 2;
+	regmatch_t      pmatch[nmatch];
+
+	int i;
+
+	ENTER("'%s %s'",string, pattern);
+
+	if (regcomp(&re, pattern, REG_EXTENDED) != 0) {
+		status = 0;
+		goto out;
+	}
+	status = regexec(&re, string, nmatch, pmatch, 0);
+	if (status != 0) {
+		regfree(&re);
+		status = 0;
+		goto out;
+	} else {
+
+
+		for (i = 0; i < nmatch; i++) {
+			int sub_string_len;
+			sub_string_len = pmatch[i].rm_eo - pmatch[i].rm_so;
+			if (i==1) {
+
+				memcpy(matches_arr[i].value, string + pmatch[i].rm_so, 
+					sub_string_len);
+				matches_arr[i].value[sub_string_len] = '\0';
+			}
+		}
+		status = 1;
+		regfree(&re);
+	}
+	out:
+	return status;
+
+}
+
+
+int sa_amf_grep(const char *string, char *pattern, size_t nmatch,
+	SaNameT *matches_arr)
+{
+	int             status;
+	regex_t         re;
+	regmatch_t      pmatch[nmatch];
+
+	int i;
+
+	ENTER("'%s %s'",string, pattern);
+	if (regcomp(&re, pattern, REG_EXTENDED) != 0) {
+		status = 0;
+		goto out;
+	}
+	status = regexec(&re, string, nmatch, pmatch, 0);
+	if (status != 0) {
+		regfree(&re);
+		status = 0;
+		goto out;
+	} else {
+
+
+		for (i = 0; i < nmatch; i++) {
+			int sub_string_len;
+			sub_string_len = pmatch[i].rm_eo - pmatch[i].rm_so; 
+			memcpy(matches_arr[i].value, string + pmatch[i].rm_so, 
+				sub_string_len);
+			matches_arr[i].value[sub_string_len] = '\0';
+			matches_arr[i].length = sub_string_len;
+		}
+		status = 1;
+		regfree(&re);
+	}
+out:
+	return status;
+
+}
+
+/**
+ * Multicast a message to the cluster. Errors are treated as
+ * fatal and will exit the program.
+ * @param msg_id
+ * @param buf
+ * @param len
+ * 
+ * @return int
+ */
+int amf_msg_mcast (int msg_id, void *buf, size_t len)
+{
+	struct req_exec_amf_msg msg;
+	struct iovec iov[2];
+	int iov_cnt;
+	int res;
+
+//	ENTER ("%u, %p, %u", msg_id, buf, len);
+
+	msg.header.size = sizeof (msg);
+	msg.header.id = SERVICE_ID_MAKE (AMF_SERVICE, msg_id);
+	iov[0].iov_base = (char *)&msg;
+	iov[0].iov_len  = sizeof (msg);
+
+	if (buf == NULL) {
+		msg.header.size = sizeof (msg);
+		iov_cnt = 1;
+	} else {
+		msg.header.size = sizeof (msg) + len;
+		iov[1].iov_base = buf;
+		iov[1].iov_len  = len;
+		iov_cnt = 2;
+	}
+
+	res = totempg_groups_mcast_joined (
+		openais_group_handle, iov, iov_cnt, TOTEMPG_AGREED);
+
+	if (res != 0) {
+		dprintf("Unable to send %d bytes\n", msg.header.size);
+		openais_exit_error (AIS_DONE_FATAL_ERR);
+	}
+
+	return res;
+}
+
+void amf_util_init (void)
+{
+	log_init ("AMF");
+}
+
+void amf_fifo_put (int entry_type, amf_fifo_t **root, int size_of_data, 
+	void *data)
+{
+	amf_fifo_t *fifo;
+	amf_fifo_t **new_item = root;
+
+	/* Insert newest entry at the end of the single linked list */
+	for (fifo = *root; fifo != NULL; fifo = fifo->next) {
+		if (fifo->next == NULL) {
+			new_item = &fifo->next;
+		}		
+	}
+	*new_item = amf_malloc (size_of_data + sizeof (amf_fifo_t));
+	fifo = *new_item;
+	
+	/* Set data of this entry*/
+	fifo->entry_type = entry_type;
+	fifo->next = NULL;
+	fifo->size_of_data = size_of_data;
+	memcpy (fifo->data, data, size_of_data);
+}
+
+int amf_fifo_get (amf_fifo_t **root, void *data)
+{
+	amf_fifo_t *fifo;
+	int result = 0;
+
+	fifo = *root;
+	if (fifo != NULL) {
+		/* Unlink oldest entry*/
+		*root = fifo->next;
+		memcpy (data, fifo->data, fifo->size_of_data);
+		free (fifo);
+		result = 1;
+	}
+	return result;
+}
+
+
+/**
+ *
+ * Use timer to call function f (void *data) after that current
+ * execution in this thread has been re-assumed because of a
+ * time-out. Time-out time is 0 msec so f will be called as soon
+ * as possible. *
+ * 
+ * @param async_func
+ * @param func_param
+ */
+
+void amf_call_function_asynchronous (async_func_t async_func, void *func_param)
+{
+	
+	static poll_timer_handle async_func_timer_handle;
+	poll_timer_add (aisexec_poll_handle, 0, func_param, async_func, 
+		&async_func_timer_handle);
+}
+
