@@ -1,12 +1,12 @@
 /*
  * Copyright (c) 2004-2006 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Red Hat, Inc.
  * Copyright (c) 2006 Sun Microsystems, Inc.
  *
- * Author: Steven Dake (sdake@redhat.com)
- *
  * All rights reserved.
  *
+ * Author: Steven Dake (sdake@mvista.com)
+ *
  * This software licensed under BSD license, the text of which follows:
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -142,7 +142,7 @@ static struct openais_exec_handler evs_exec_service[] =
 };
 
 struct openais_service_handler evs_service_handler = {
-	.name			= (unsigned char*)"openais extended virtual synchrony service",
+	.name			= "openais extended virtual synchrony service",
 	.id			= EVS_SERVICE,
 	.private_data_size	= sizeof (struct evs_pd),
 	.flow_control		= OPENAIS_FLOW_CONTROL_REQUIRED, 
@@ -291,7 +291,7 @@ static void message_handler_req_evs_join (void *conn, void *msg)
 
 	addr = realloc (evs_pd->groups, sizeof (struct evs_group) * 
 		(evs_pd->group_entries + req_lib_evs_join->group_entries));
-	if (addr == 0) {
+	if (addr == NULL) {
 		error = SA_AIS_ERR_NO_MEMORY;
 		goto exit_error;
 	}
@@ -378,11 +378,11 @@ static void message_handler_req_evs_mcast_joined (void *conn, void *msg)
 	req_exec_evs_mcast.msg_len = req_lib_evs_mcast_joined->msg_len;
 	req_exec_evs_mcast.group_entries = evs_pd->group_entries;
 
-	req_exec_evs_mcast_iovec[0].iov_base = &req_exec_evs_mcast;
+	req_exec_evs_mcast_iovec[0].iov_base = (char *)&req_exec_evs_mcast;
 	req_exec_evs_mcast_iovec[0].iov_len = sizeof (req_exec_evs_mcast);
-	req_exec_evs_mcast_iovec[1].iov_base = evs_pd->groups;
+	req_exec_evs_mcast_iovec[1].iov_base = (char *)evs_pd->groups;
 	req_exec_evs_mcast_iovec[1].iov_len = evs_pd->group_entries * sizeof (struct evs_group);
-	req_exec_evs_mcast_iovec[2].iov_base = &req_lib_evs_mcast_joined->msg;
+	req_exec_evs_mcast_iovec[2].iov_base = (char *)&req_lib_evs_mcast_joined->msg;
 	req_exec_evs_mcast_iovec[2].iov_len = req_lib_evs_mcast_joined->msg_len;
 // TODO this doesn't seem to work for some reason	
 	send_ok = totempg_groups_send_ok_joined (openais_group_handle, req_exec_evs_mcast_iovec, 3);
@@ -425,9 +425,9 @@ static void message_handler_req_evs_mcast_groups (void *conn, void *msg)
 		sizeof (struct req_lib_evs_mcast_groups) + 
 		(sizeof (struct evs_group) * req_lib_evs_mcast_groups->group_entries);
 
-	req_exec_evs_mcast_iovec[0].iov_base = &req_exec_evs_mcast;
+	req_exec_evs_mcast_iovec[0].iov_base = (char *)&req_exec_evs_mcast;
 	req_exec_evs_mcast_iovec[0].iov_len = sizeof (req_exec_evs_mcast);
-	req_exec_evs_mcast_iovec[1].iov_base = &req_lib_evs_mcast_groups->groups;
+	req_exec_evs_mcast_iovec[1].iov_base = (char *)&req_lib_evs_mcast_groups->groups;
 	req_exec_evs_mcast_iovec[1].iov_len = sizeof (struct evs_group) * req_lib_evs_mcast_groups->group_entries;
 	req_exec_evs_mcast_iovec[2].iov_base = msg_addr;
 	req_exec_evs_mcast_iovec[2].iov_len = req_lib_evs_mcast_groups->msg_len;
