@@ -1,10 +1,10 @@
 /*
  * Copyright (c) 2002-2006 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Red Hat, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * This software licensed under BSD license, the text of which follows:
  *
@@ -54,6 +54,9 @@
 #include <signal.h>
 #include <sched.h>
 #include <time.h>
+#if defined(OPENAIS_SOLARIS) && defined(HAVE_GETPEERUCRED)
+#include <ucred.h>
+#endif
 
 #include "../include/saAis.h"
 #include "../include/list.h"
@@ -79,6 +82,10 @@
 
 #include "util.h"
 
+#ifdef OPENAIS_SOLARIS
+#define MSG_NOSIGNAL 0
+#endif
+
 #define SERVER_BACKLOG 5
 
 /*
@@ -470,20 +477,8 @@ static void *prioritized_poll_thread (void *conn)
 	struct conn_info *cinfo_partner;
 	void *private_data;
 
-#if defined(OPENAIS_BSD) || defined(OPENAIS_LINUX)
-	res = sched_get_priority_max (SCHED_RR);
-	if (res != -1) {
-		sched_param.sched_priority = res;
-		res = pthread_setschedparam (conn_info->thread, SCHED_RR, &sched_param);
-		if (res == -1) {
-			log_printf (LOG_LEVEL_WARNING, "Could not set SCHED_RR at priority %d: %s\n",
-				sched_param.sched_priority, strerror (errno));
-		}
-	} else
-		log_printf (LOG_LEVEL_WARNING, "Could not get maximum scheduler priority: %s\n", strerror (errno));
-#else
-	log_printf(LOG_LEVEL_WARNING, "Scheduler priority left to default value (no OS support)\n");
-#endif
+	sched_param.sched_priority = 1;
+	res = pthread_setschedparam (conn_info->thread, SCHED_RR, &sched_param);
 
 	ufd.fd = conn_info->fd;
 	for (;;) {
@@ -577,15 +572,17 @@ retry_poll:
 	return (0);
 }
 
-#if defined(OPENAIS_LINUX)
+#if defined(OPENAIS_LINUX) || defined(OPENAIS_SOLARIS)
 /* SUN_LEN is broken for abstract namespace
  */
 #define AIS_SUN_LEN(a) sizeof(*(a))
-
-char *socketname = "libais.socket";
 #else
 #define AIS_SUN_LEN(a) SUN_LEN(a)
+#endif
 
+#if defined(OPENAIS_LINUX)
+char *socketname = "libais.socket";
+#else
 char *socketname = "/var/run/libais.socket";
 #endif
 
@@ -665,9 +662,14 @@ static int conn_info_outq_flush (struct conn_info *conn_info) {
 	msg_send.msg_name = 0;
 	msg_send.msg_namelen = 0;
 	msg_send.msg_iovlen = 1;
+#ifndef OPENAIS_SOLARIS
 	msg_send.msg_control = 0;
 	msg_send.msg_controllen = 0;
 	msg_send.msg_flags = 0;
+#else
+	msg_send.msg_accrights = 0;
+	msg_send.msg_accrightslen = 0;
+#endif
 
 	while (!queue_is_empty (outq)) {
 		queue_item = queue_item_get (outq);
@@ -733,9 +735,6 @@ static void libais_deliver (struct conn_info *conn_info)
 	char cmsg_cred[CMSG_SPACE (sizeof (struct ucred))];
 	struct ucred *cred;
 	int on = 0;
-#else
-	uid_t euid;
-	gid_t egid;
 #endif
 	int send_ok = 0;
 	int send_ok_joined = 0;
@@ -746,6 +745,7 @@ static void libais_deliver (struct conn_info *conn_info)
 	msg_recv.msg_iovlen = 1;
 	msg_recv.msg_name = 0;
 	msg_recv.msg_namelen = 0;
+#ifndef OPENAIS_SOLARIS
 	msg_recv.msg_flags = 0;
 
 	if (conn_info->authenticated) {
@@ -756,10 +756,11 @@ static void libais_deliver (struct conn_info *conn_info)
 		msg_recv.msg_control = (void *)cmsg_cred;
 		msg_recv.msg_controllen = sizeof (cmsg_cred);
 #else
-		euid = -1; egid = -1;
+		uid_t euid = -1;
+		gid_t egid = -1;
 		if (getpeereid(conn_info->fd, &euid, &egid) != -1 &&
 		    (euid == 0 || egid == g_gid_valid)) {
-				conn_info->authenticated = 1;
+			conn_info->authenticated = 1;
 		}
 		if (conn_info->authenticated == 0) {
 			log_printf (LOG_LEVEL_SECURITY, "Connection not authenticated because gid is %d, expecting %d\n", egid, g_gid_valid);
@@ -767,6 +768,36 @@ static void libais_deliver (struct conn_info *conn_info)
 #endif
 	}
 
+#else	/* OPENAIS_SOLARIS */
+	msg_recv.msg_accrights = 0;
+	msg_recv.msg_accrightslen = 0;
+
+	if (! conn_info->authenticated) {
+#ifdef HAVE_GETPEERUCRED
+		ucred_t *uc;
+		uid_t euid = -1;
+		gid_t egid = -1;
+		if (getpeerucred(conn_info->fd, &uc) == 0) {
+			euid = ucred_geteuid(uc);
+			egid = ucred_getegid(uc);
+			if ((euid == 0) || (egid == g_gid_valid)) {
+				conn_info->authenticated = 1;
+			}
+			ucred_free(uc);
+		}
+		if (conn_info->authenticated == 0) {
+			log_printf (LOG_LEVEL_SECURITY, "Connection not authenticated because gid is %d, expecting %d\n", (int)egid, g_gid_valid);
+		}
+#else
+		log_printf (LOG_LEVEL_SECURITY, "Connection not authenticated "
+				"because platform does not support "
+				"authentication with sockets, continuing "
+				"with a fake authentication\n");
+		conn_info->authenticated = 1;
+#endif
+	}
+#endif
+
 	iov_recv.iov_base = &conn_info->inb[conn_info->inb_start];
 	iov_recv.iov_len = (SIZEINB) - conn_info->inb_start;
 	if (conn_info->inb_inuse == SIZEINB) {
@@ -847,7 +878,7 @@ retry_recv:
 			 * to queue a message, otherwise tell the library we are busy and to
 			 * try again later
 			 */
-			send_ok_joined_iovec.iov_base = header;
+			send_ok_joined_iovec.iov_base = (char *)header;
 			send_ok_joined_iovec.iov_len = header->size;
 			send_ok_joined = totempg_groups_send_ok_joined (openais_group_handle,
 				&send_ok_joined_iovec, 1);
@@ -972,7 +1003,6 @@ void message_source_set (
 	void *conn)
 {
 	assert ((source != NULL) && (conn != NULL));
-	memset (source, 0, sizeof (mar_message_source_t));
 	source->nodeid = my_ip->nodeid;
 	source->conn = conn;
 }
@@ -1120,9 +1150,14 @@ int openais_conn_send_response (
 	msg_send.msg_name = 0;
 	msg_send.msg_namelen = 0;
 	msg_send.msg_iovlen = 1;
+#ifndef OPENAIS_SOLARIS
 	msg_send.msg_control = 0;
 	msg_send.msg_controllen = 0;
 	msg_send.msg_flags = 0;
+#else
+	msg_send.msg_accrights = 0;
+	msg_send.msg_accrightslen = 0;
+#endif
 
 	if (queue_is_full (outq)) {
 		/*
@@ -1225,6 +1260,44 @@ retry_sendmsg_two:
 	return (0);
 }
 
+int openais_ipc_timer_add (
+	void *conn,
+	void (*timer_fn) (void *data),
+	void *data,
+	unsigned int msec_in_future,
+	timer_handle *handle)
+{
+	struct conn_info *conn_info = (struct conn_info *)conn;
+	int res;
+
+	res = timerlist_add_future (
+		&conn_info->timerlist,
+		timer_fn,
+		data,
+		msec_in_future,
+		handle);
+
+	return (res);
+}
+
+void openais_ipc_timer_del (
+	void *conn,
+	timer_handle timer_handle)
+{
+	struct conn_info *conn_info = (struct conn_info *)conn;
+
+	timerlist_del (&conn_info->timerlist, timer_handle);
+}
+
+void openais_ipc_timer_del_data (
+	void *conn,
+	timer_handle timer_handle)
+{
+	struct conn_info *conn_info = (struct conn_info *)conn;
+
+	timerlist_del (&conn_info->timerlist, timer_handle);
+}
+
 void openais_ipc_flow_control_create (
 	void *conn,
 	unsigned int service,
