@@ -1,10 +1,9 @@
 /*
  * Copyright (c) 2002-2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * This software licensed under BSD license, the text of which follows:
  *
@@ -205,10 +204,14 @@ int openais_main_config_read (
 									"logger",
 									strlen ("logger"),
 									&object_logger_handle) == 0) {
-			main_config->logger =
-				realloc(main_config->logger,
-						sizeof(struct logger_config) *
-						(main_config->loggers + 1));
+			struct logger_config *logger_tmp;
+			logger_tmp = realloc (main_config->logger,
+				sizeof(struct logger_config) * (main_config->loggers + 1));
+			if (logger_tmp == NULL) {
+				error_reason = "no more memory";
+				goto other_error;
+			}
+			main_config->logger = logger_tmp;
 			i = main_config->loggers;
 			main_config->loggers++;
 			memset(&main_config->logger[i], 0, sizeof(struct logger_config));
@@ -281,13 +284,11 @@ int openais_main_config_read (
 	}
 
 	/* Default user/group */
-	if (main_config->user == NULL) {
+	if (!main_config->user)
 		main_config->user = "ais";
-	}
 
-	if (main_config->group == NULL) {
+	if (!main_config->group)
 		main_config->group = "ais";
-	}
 
 	if ((main_config->logmode & LOG_MODE_FILE) && main_config->logfile == 0) {
 		error_reason = "logmode set to 'file' but no logfile specified";
@@ -299,6 +300,14 @@ int openais_main_config_read (
 
 	return 0;
 
+other_error:
+	sprintf (error_string_response,
+		"error parsing config: %s.\n",
+		error_reason);
+
+	*error_string = error_string_response;
+	return -1;
+
 parse_error:
 	sprintf (error_string_response,
 		 "parse error in config: %s.\n",
