@@ -1,10 +1,10 @@
 /*
  * Copyright (c) 2005-2006 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Red Hat, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * This software licensed under BSD license, the text of which follows:
  *
@@ -433,7 +433,7 @@ static struct openais_exec_handler msg_exec_service[] = {
 };
 
 struct openais_service_handler msg_service_handler = {
-	.name				= (unsigned char *)"openais message service B.01.01",
+	.name				= "openais message service B.01.01",
 	.id				= MSG_SERVICE,
 	.private_data_size		= sizeof (struct msg_pd),
 	.flow_control			= OPENAIS_FLOW_CONTROL_NOT_REQUIRED, 
