@@ -1,9 +1,9 @@
 /*
  * Copyright (c) 2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Red Hat, Inc.
  * Copyright (c) 2006 Sun Microsystems, Inc.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * All rights reserved.
  *
@@ -36,6 +36,7 @@
 #ifndef TOTEM_H_DEFINED
 #define TOTEM_H_DEFINED
 #include "totemip.h"
+#include "../include/swab.h"
 
 #define MESSAGE_SIZE_MAX	1024*1024 /* (1MB) */
 #define PROCESSOR_COUNT_MAX	384
@@ -164,5 +165,26 @@ struct memb_ring_id {
 	unsigned long long seq;
 } __attribute__((packed));
 
+typedef struct memb_ring_id memb_ring_id_t;
+ 
+static inline void swab_memb_ring_id_t (memb_ring_id_t *to_swab)
+{
+	swab_totem_ip_address_t (&to_swab->rep);
+	to_swab->seq = swab64 (to_swab->seq);
+}
+
+static inline void memb_ring_id_copy(
+	memb_ring_id_t *out, memb_ring_id_t *in)
+{
+	totemip_copy (&out->rep, &in->rep);
+	out->seq = in->seq;
+}
+
+static inline void memb_ring_id_copy_endian_convert(
+	memb_ring_id_t *out, memb_ring_id_t *in)
+{
+	totemip_copy_endian_convert (&out->rep, &in->rep);
+	out->seq = swab64 (in->seq);
+}
 
 #endif /* TOTEM_H_DEFINED */
