@@ -2,12 +2,11 @@
  * Copyright (c) 2003-2005 MontaVista Software, Inc.
  * Copyright (c) 2005 OSDL.
  * Copyright (c) 2006 Sun Microsystems, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
- * Author: Mark Haverkamp (markh@osdl.org)
+ * Author: Steven Dake (sdake@mvista.com)
+ *         Mark Haverkamp (markh@osdl.org)
  *
  * This software licensed under BSD license, the text of which follows:
  * 
@@ -83,6 +82,9 @@
  *	
  */
 
+#ifndef OPENAIS_BSD
+#include <alloca.h>
+#endif
 #include <netinet/in.h>
 #include <sys/uio.h>
 #include <stdio.h>
@@ -178,7 +180,7 @@ DECLARE_LIST_INIT(assembly_list_free);
  * fragment_contuation indicates whether the first packed message in 
  * the buffer is a continuation of a previously packed fragment.
  */
-static unsigned char *fragmentation_data;
+static char *fragmentation_data;
 
 static int fragment_size = 0;
 
@@ -337,25 +339,25 @@ static inline int group_matches (
 	char *group_name;
 	int i;
 	int j;
-        struct iovec iovec_aligned = { NULL, 0 };
+#ifdef __sparc
+	struct iovec iovec_aligned = { NULL, 0 };
+#endif
 	
 	assert (iov_len == 1);
 
-	/*
-	 * Align data structure for sparc and ia64
-	 */
+#ifdef __sparc
 	if ((size_t)iovec->iov_base % 4 != 0) {
 		iovec_aligned.iov_base = alloca(iovec->iov_len);
-		memcpy(iovec_aligned.iov_base, iovec->iov_base, iovec->iov_len);                iovec_aligned.iov_len = iovec->iov_len;
+		memcpy(iovec_aligned.iov_base, iovec->iov_base, iovec->iov_len);
+		iovec_aligned.iov_len = iovec->iov_len;
 		iovec = &iovec_aligned;
 	}
-
+#endif
 
 	group_len = (unsigned short *)iovec->iov_base;
 	group_name = ((char *)iovec->iov_base) +
 		sizeof (unsigned short) * (group_len[0] + 1);
 
-
 	/*
 	 * Calculate amount to adjust the iovec by before delivering to app
 	 */
@@ -389,46 +391,53 @@ static inline void app_deliver_fn (
 	int i;
 	struct totempg_group_instance *instance;
 	struct iovec stripped_iovec;
+#ifdef __sparc
+	struct iovec aligned_iovec = { NULL, 0 };
+#endif
 	unsigned int adjust_iovec;
 	unsigned int res;
-        struct iovec aligned_iovec = { NULL, 0 };
 
 	if (endian_conversion_required) {
+#ifdef __sparc
+		if ((size_t)iovec->iov_base % 4 != 0) {
+			/* Deal with misalignment */
+			aligned_iovec.iov_base = alloca(iovec->iov_len);
+			aligned_iovec.iov_len = iovec->iov_len;
+			memcpy(aligned_iovec.iov_base, iovec->iov_base,
+				iovec->iov_len);
+			iovec = &aligned_iovec;
+		}
+#endif
 		group_endian_convert (iovec);
 	}
-
-	/*
-	 * Align data structure for sparc and ia64
-	 */
-	aligned_iovec.iov_base = alloca(iovec->iov_len);
-	aligned_iovec.iov_len = iovec->iov_len;
-	memcpy(aligned_iovec.iov_base, iovec->iov_base, iovec->iov_len);
-	iovec = &aligned_iovec;
-
 	for (i = 0; i <= totempg_max_handle; i++) {
 		res = hdb_handle_get (&totempg_groups_instance_database,
 			i, (void *)&instance);
 
 		if (res == 0) {
 			assert (iov_len == 1);
-			if (group_matches (iovec, iov_len, instance->groups, instance->groups_cnt, &adjust_iovec)) {
-				stripped_iovec.iov_len = iovec->iov_len - adjust_iovec;
-//				stripped_iovec.iov_base = (char *)iovec->iov_base + adjust_iovec;
-
-				/*
-				 * Align data structure for sparc and ia64
-				 */
+			if (group_matches (iovec, iov_len, instance->groups,
+				instance->groups_cnt, &adjust_iovec)) {
+				stripped_iovec.iov_len =
+					iovec->iov_len - adjust_iovec;
+#ifndef __sparc
+				stripped_iovec.iov_base =
+					(char *)iovec->iov_base + adjust_iovec;
+#else
 				if (iovec->iov_base + adjust_iovec % 4 != 0) {
+					/* Deal with misalignment */
 					/*
-					 * Deal with misalignment
+					 * XXX Using alloca() is dangerous,
+					 * since it may be called multiple
+					 * times within the for() loop
 					 */
-					stripped_iovec.iov_base =
-						alloca (stripped_iovec.iov_len);
-					memcpy (stripped_iovec.iov_base,
-						 iovec->iov_base + adjust_iovec,
+					stripped_iovec.iov_base = alloca(
+						stripped_iovec.iov_len);
+					memcpy(stripped_iovec.iov_base,
+						iovec->iov_base + adjust_iovec,
 						stripped_iovec.iov_len);
 				}
-
+#endif
 				instance->deliver_fn (
 					nodeid,
 					&stripped_iovec,
@@ -440,7 +449,6 @@ static inline void app_deliver_fn (
 		}
 	}
 }
-
 static void totempg_confchg_fn (
 	enum totem_configuration_type configuration_type,
 	unsigned int *member_list, int member_list_entries,
@@ -496,7 +504,9 @@ static void totempg_deliver_fn (
 		msg_count = mcast->msg_count;
 		datasize = sizeof (struct totempg_mcast) +
 			msg_count * sizeof (unsigned short);
-		
+
+		assert (iovec[0].iov_len >= datasize);
+
 		memcpy (header, iovec[0].iov_base, datasize);
 		assert(iovec);
 		data = iovec[0].iov_base;
@@ -513,7 +523,7 @@ static void totempg_deliver_fn (
 	} else {
 		/* 
 		 * The message originated from local processor  
-		 * becasue there is greater than one iovec for then full msg.
+		 * because there is greater than one iovec for then full msg.
 		 */
 		h_index = 0;
 		for (i = 0; i < 2; i++) {
@@ -543,7 +553,7 @@ static void totempg_deliver_fn (
 	 */
 	msg_count = mcast->fragmented ? mcast->msg_count - 1 : mcast->msg_count;
 	continuation = mcast->continuation;
-	iov_delv.iov_base = &assembly->data[0];
+	iov_delv.iov_base = (char *)&assembly->data[0];
 	iov_delv.iov_len = assembly->index + msg_lens[0];
 
 	/*
@@ -580,7 +590,7 @@ static void totempg_deliver_fn (
 		 */
 		if (!continuation) {
 			assembly->index += msg_lens[0];
-			iov_delv.iov_base = &assembly->data[assembly->index];
+			iov_delv.iov_base = (char *)&assembly->data[assembly->index];
 			iov_delv.iov_len = msg_lens[1];
 			start = 1;
 		}
@@ -591,7 +601,7 @@ static void totempg_deliver_fn (
 		app_deliver_fn(nodeid, &iov_delv, 1,
 			endian_conversion_required);
 		assembly->index += msg_lens[i];
-		iov_delv.iov_base = &assembly->data[assembly->index];
+		iov_delv.iov_base = (char *)&assembly->data[assembly->index];
 		if (i < (msg_count - 1)) {
 			iov_delv.iov_len = msg_lens[i + 1];
 		}
@@ -654,9 +664,9 @@ int callback_token_received_fn (enum totem_callback_token_type type,
 
 	mcast.msg_count = mcast_packed_msg_count;
 
-	iovecs[0].iov_base = &mcast;
+	iovecs[0].iov_base = (char *)&mcast;
 	iovecs[0].iov_len = sizeof (struct totempg_mcast);
-	iovecs[1].iov_base = mcast_packed_msg_lens;
+	iovecs[1].iov_base = (char *)mcast_packed_msg_lens;
 	iovecs[1].iov_len = mcast_packed_msg_count * sizeof (unsigned short);
 	iovecs[2].iov_base = &fragmentation_data[0];
 	iovecs[2].iov_len = fragment_size;
@@ -782,7 +792,7 @@ static int mcast_msg (
 		 * If it just fits or is too big, then send out what fits.
 		 */
 		} else {
-			unsigned char *data_ptr;
+			char *data_ptr;
 
 			copy_len = min(copy_len, max_packet_size - fragment_size);
 			if( copy_len == max_packet_size )
@@ -819,9 +829,9 @@ static int mcast_msg (
 			 * assemble the message and send it
 			 */
 			mcast.msg_count = ++mcast_packed_msg_count;
-			iovecs[0].iov_base = &mcast;
+			iovecs[0].iov_base = (char *)&mcast;
 			iovecs[0].iov_len = sizeof(struct totempg_mcast);
-			iovecs[1].iov_base = mcast_packed_msg_lens;
+			iovecs[1].iov_base = (char *)mcast_packed_msg_lens;
 			iovecs[1].iov_len = mcast_packed_msg_count * 
 				sizeof(unsigned short);
 			iovecs[2].iov_base = data_ptr;
@@ -988,7 +998,7 @@ int totempg_groups_join (
 	new_groups = realloc (instance->groups,
 		sizeof (struct totempg_group) *
 		(instance->groups_cnt + group_cnt));
-	if (new_groups == 0) {
+	if (new_groups == NULL) {
 		res = ENOMEM;
 		goto error_exit;
 	}
@@ -1058,7 +1068,7 @@ int totempg_groups_mcast_joined (
 		iovec_mcast[i + 1].iov_base = instance->groups[i].group;
 	}
 	iovec_mcast[0].iov_len = (instance->groups_cnt + 1) * sizeof (unsigned short);
-	iovec_mcast[0].iov_base = group_len;
+	iovec_mcast[0].iov_base = (char *)group_len;
 	for (i = 0; i < iov_len; i++) {
 		iovec_mcast[i + instance->groups_cnt + 1].iov_len = iovec[i].iov_len;
 		iovec_mcast[i + instance->groups_cnt + 1].iov_base = iovec[i].iov_base;
@@ -1138,7 +1148,7 @@ int totempg_groups_mcast_groups (
 		iovec_mcast[i + 1].iov_base = groups[i].group;
 	}
 	iovec_mcast[0].iov_len = (groups_cnt + 1) * sizeof (unsigned short);
-	iovec_mcast[0].iov_base = group_len;
+	iovec_mcast[0].iov_base = (char *)group_len;
 	for (i = 0; i < iov_len; i++) {
 		iovec_mcast[i + groups_cnt + 1].iov_len = iovec[i].iov_len;
 		iovec_mcast[i + groups_cnt + 1].iov_base = iovec[i].iov_base;
