@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -142,18 +142,6 @@ cpg_error_t cpg_fd_get (
 	cpg_handle_t handle,
 	int *fd);
 
-/* 
- * Get and set contexts for a CPG handle
- */
-cpg_error_t cpg_context_get (
-	cpg_handle_t handle,
-	void **context);
-
-cpg_error_t cpg_context_set (
-	cpg_handle_t handle,
-	void *context);
-
-
 /*
  * Dispatch messages and configuration changes
  */
@@ -199,10 +187,6 @@ cpg_error_t cpg_membership_get (
 	struct cpg_address *member_list,
 	int *member_list_entries);
 
-cpg_error_t cpg_local_get (
-	cpg_handle_t handle,
-	unsigned int *local_nodeid);
-
 cpg_error_t cpg_flow_control_state_get (
 	cpg_handle_t handle,
 	cpg_flow_control_state_t *flow_control_enabled);
