@@ -1,9 +1,10 @@
 /*
- * Copyright (C) 2006-2007 Red Hat, Inc.
+ * Copyright (C) 2006 Red Hat, Inc.
+ * Copyright (c) 2006 Sun Microsystems, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * This software licensed under BSD license, the text of which follows:
  * 
@@ -35,7 +36,11 @@
 #ifndef AIS_MAR_GEN_H_DEFINED
 #define AIS_MAR_GEN_H_DEFINED
 
+#ifndef OPENAIS_SOLARIS
 #include <stdint.h>
+#else
+#include <sys/types.h>
+#endif
 #include <string.h>
 
 #include "saAis.h"
