@@ -1,10 +1,10 @@
 /*
  * Copyright (c) 2002-2003 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Sun Microsystems, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * This software licensed under BSD license, the text of which follows:
  * 
@@ -45,7 +45,11 @@ typedef enum {
 	SA_TRUE = 1
 } SaBoolT;
 
+#ifndef OPENAIS_SOLARIS
 #include <stdint.h>
+#else
+#include <sys/types.h>
+#endif
 
 typedef int8_t SaInt8T;
 typedef int16_t SaInt16T;
