@@ -1,10 +1,10 @@
+
 /*
  * Copyright (c) 2002-2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * This software licensed under BSD license, the text of which follows:
  * 
@@ -191,13 +191,16 @@ saAmfDispatch (
 	struct res_lib_amf_healthcheckcallback *res_lib_amf_healthcheckcallback;
 	struct res_lib_amf_csiremovecallback *res_lib_amf_csiremovecallback;
 	struct res_lib_amf_componentterminatecallback *res_lib_amf_componentterminatecallback;
-
-	/*
-	struct res_lib_amf_protectiongrouptrackcallback *res_lib_amf_protectiongrouptrackcallback;
-	*/
 	SaAmfCallbacksT callbacks;
 	struct res_overlay dispatch_data;
 
+	if (dispatchFlags != SA_DISPATCH_ONE &&
+		dispatchFlags != SA_DISPATCH_ALL &&
+		dispatchFlags != SA_DISPATCH_BLOCKING) {
+
+		return (SA_AIS_ERR_INVALID_PARAM);
+	}
+
 	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
 		(void *)&amfInstance);
 	if (error != SA_AIS_OK) {
@@ -221,23 +224,21 @@ saAmfDispatch (
 
 		error = saPollRetry (&ufds, 1, timeout);
 		if (error != SA_AIS_OK) {
-			goto error_nounlock;
+			goto error_put;
 		}
 
 		pthread_mutex_lock (&amfInstance->dispatch_mutex);
 
-		error = saPollRetry (&ufds, 1, 0);
-		if (error != SA_AIS_OK) {
-			goto error_nounlock;
-		}
-
 		/*
 		 * Handle has been finalized in another thread
 		 */
 		if (amfInstance->finalize == 1) {
 			error = SA_AIS_OK;
-			pthread_mutex_unlock (&amfInstance->dispatch_mutex);
+			goto error_unlock;
+		}
 
+		if ((ufds.revents & (POLLERR|POLLHUP|POLLNVAL)) != 0) {
+			error = SA_AIS_ERR_BAD_HANDLE;
 			goto error_unlock;
 		}
 
@@ -378,7 +379,7 @@ saAmfDispatch (
 			break;
 		default:
 			error = SA_AIS_ERR_LIBRARY;	
-			goto error_nounlock;
+			goto error_put;
 			break;
 		}
 
@@ -397,8 +398,10 @@ saAmfDispatch (
 	} while (cont);
 
 error_unlock:
+	pthread_mutex_unlock (&amfInstance->dispatch_mutex);
+error_put:
 	saHandleInstancePut (&amfHandleDatabase, amfHandle);
-error_nounlock:
+
 	return (error);
 }
 
@@ -431,11 +434,9 @@ saAmfFinalize (
 	amfInstance->finalize = 1;
 
 	pthread_mutex_unlock (&amfInstance->response_mutex);
-	pthread_mutex_destroy (&amfInstance->response_mutex);
 
 	pthread_mutex_unlock (&amfInstance->dispatch_mutex);
-	pthread_mutex_destroy (&amfInstance->dispatch_mutex);
-	
+
 	saHandleDestroy (&amfHandleDatabase, amfHandle);
 
 	if (amfInstance->response_fd != -1) {
@@ -601,7 +602,7 @@ saAmfPmStart (
 		return (error);
 	}
 
-	req_lib_amf_pmstart.header.size = sizeof (struct req_lib_amf_componentunregister);
+	req_lib_amf_pmstart.header.size = sizeof (struct req_lib_amf_pmstart);
 	req_lib_amf_pmstart.header.id = MESSAGE_REQ_AMF_PMSTART;
 	memcpy (&req_lib_amf_pmstart.compName, compName,
 		sizeof (SaNameT));
@@ -727,7 +728,7 @@ saAmfHealthcheckConfirm (
 		return (error);
 	}
 
-	req_lib_amf_healthcheckconfirm.header.size = sizeof (struct req_lib_amf_componentunregister);
+	req_lib_amf_healthcheckconfirm.header.size = sizeof (struct req_lib_amf_healthcheckconfirm);
 	req_lib_amf_healthcheckconfirm.header.id = MESSAGE_REQ_AMF_HEALTHCHECKCONFIRM;
 	memcpy (&req_lib_amf_healthcheckconfirm.compName, compName,
 		sizeof (SaNameT));
@@ -847,7 +848,7 @@ saAmfCSIQuiescingComplete (
 		return (error);
 	}
 
-	req_lib_amf_csiquiescingcomplete.header.size = sizeof (struct req_lib_amf_componentunregister);
+	req_lib_amf_csiquiescingcomplete.header.size = sizeof (struct req_lib_amf_csiquiescingcomplete);
 	req_lib_amf_csiquiescingcomplete.header.id = MESSAGE_REQ_AMF_CSIQUIESCINGCOMPLETE;
 	req_lib_amf_csiquiescingcomplete.invocation = invocation;
 	req_lib_amf_csiquiescingcomplete.error = error;
@@ -966,7 +967,7 @@ saAmfComponentErrorReport (
 	memcpy (&req_lib_amf_componenterrorreport.erroneousComponent, erroneousComponent,
 		sizeof (SaNameT));
 	req_lib_amf_componenterrorreport.errorDetectionTime = errorDetectionTime;
-
+	req_lib_amf_componenterrorreport.recommendedRecovery = recommendedRecovery;
     DPRINT (("start error report\n"));
 	error = saSendReceiveReply (amfInstance->response_fd,
 		&req_lib_amf_componenterrorreport,
