@@ -1,10 +1,10 @@
 /*
  * Copyright (c) 2002-2004 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Red Hat, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * This software licensed under BSD license, the text of which follows:
  * 
@@ -191,8 +191,6 @@ static void ckptSectionIterationInstanceFinalize (struct ckptSectionIterationIns
 
 	saHandleDestroy (&ckptSectionIterationHandleDatabase,
 		ckptSectionIterationInstance->sectionIterationHandle);
-
-	pthread_mutex_destroy (&ckptSectionIterationInstance->response_mutex);
 }
 
 static void ckptCheckpointInstanceFinalize (struct ckptCheckpointInstance *ckptCheckpointInstance)
@@ -216,8 +214,6 @@ static void ckptCheckpointInstanceFinalize (struct ckptCheckpointInstance *ckptC
 	list_del (&ckptCheckpointInstance->list);
 
 	saHandleDestroy (&checkpointHandleDatabase, ckptCheckpointInstance->checkpointHandle);
-
-	pthread_mutex_destroy (&ckptCheckpointInstance->response_mutex);
 }
 
 static void ckptInstanceFinalize (struct ckptInstance *ckptInstance)
@@ -239,9 +235,6 @@ static void ckptInstanceFinalize (struct ckptInstance *ckptInstance)
 	}
 
 	saHandleDestroy (&ckptHandleDatabase, ckptInstance->ckptHandle);
-
-	pthread_mutex_destroy (&ckptInstance->response_mutex);
-	pthread_mutex_destroy (&ckptInstance->dispatch_mutex);
 }
 
 /**
@@ -528,7 +521,7 @@ saCkptFinalize (
 	}
 
 	pthread_mutex_lock (&ckptInstance->response_mutex);
-	
+
 	/*
 	 * Another thread has already started finalizing
 	 */
@@ -541,7 +534,7 @@ saCkptFinalize (
 	ckptInstance->finalize = 1;
 
 	pthread_mutex_unlock (&ckptInstance->response_mutex);
-	
+
 	ckptInstanceFinalize (ckptInstance);
 
 	if (ckptInstance->response_fd != -1) {
@@ -1539,7 +1532,7 @@ saCkptCheckpointWrite (
 		iov_idx++;
 
 		if (ioVector[i].sectionId.idLen) {
-			iov[iov_idx].iov_base = ioVector[i].sectionId.id;
+			iov[iov_idx].iov_base = (char *)ioVector[i].sectionId.id;
 			iov[iov_idx].iov_len = ioVector[i].sectionId.idLen;
 			iov_idx++;
 		}
@@ -1706,7 +1699,7 @@ saCkptCheckpointRead (
 
 		iov[0].iov_base = (char *)&req_lib_ckpt_sectionread;
 		iov[0].iov_len = sizeof (struct req_lib_ckpt_sectionread);
-		iov[1].iov_base = ioVector[i].sectionId.id;
+		iov[1].iov_base = (char *)ioVector[i].sectionId.id;
 		iov[1].iov_len = ioVector[i].sectionId.idLen;
 
 		error = saSendMsgRetry (ckptCheckpointInstance->response_fd,
