@@ -1,7 +1,7 @@
 /*
  * vi: set autoindent tabstop=4 shiftwidth=4 :
  *
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -57,7 +57,6 @@ struct cpg_inst {
 	int finalize;
 	cpg_flow_control_state_t flow_control_state;
 	cpg_callbacks_t callbacks;
-	void *context;
 	pthread_mutex_t response_mutex;
 	pthread_mutex_t dispatch_mutex;
 };
@@ -153,9 +152,6 @@ cpg_error_t cpg_finalize (
 	cpg_inst->finalize = 1;
 
 	pthread_mutex_unlock (&cpg_inst->response_mutex);
-	pthread_mutex_destroy (&cpg_inst->response_mutex);
-
-	pthread_mutex_destroy (&cpg_inst->dispatch_mutex);
 
 	saHandleDestroy (&cpg_handle_t_db, handle);
 
@@ -194,44 +190,6 @@ cpg_error_t cpg_fd_get (
 	return (SA_AIS_OK);
 }
 
-cpg_error_t cpg_context_get (
-	cpg_handle_t handle,
-	void **context)
-{
-	SaAisErrorT error;
-	struct cpg_inst *cpg_inst;
-
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
-		return (error);
-	}
-
-	*context = cpg_inst->context;
-
-	saHandleInstancePut (&cpg_handle_t_db, handle);
-
-	return (SA_AIS_OK);
-}
-
-cpg_error_t cpg_context_set (
-	cpg_handle_t handle,
-	void *context)
-{
-	SaAisErrorT error;
-	struct cpg_inst *cpg_inst;
-
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
-		return (error);
-	}
-
-	cpg_inst->context = context;
-
-	saHandleInstancePut (&cpg_handle_t_db, handle);
-
-	return (SA_AIS_OK);
-}
-
 struct res_overlay {
 	mar_res_header_t header __attribute__((aligned(8)));
 	char data[512000];
@@ -454,7 +412,7 @@ cpg_error_t cpg_join (
 	marshall_to_mar_cpg_name_t (&req_lib_cpg_trackstart.group_name,
 		group);
 
-	iov[0].iov_base = &req_lib_cpg_trackstart;
+	iov[0].iov_base = (char *)&req_lib_cpg_trackstart;
 	iov[0].iov_len = sizeof (struct req_lib_cpg_trackstart);
 
 	error = saSendMsgReceiveReply (cpg_inst->dispatch_fd, iov, 1,
@@ -472,7 +430,7 @@ cpg_error_t cpg_join (
 	marshall_to_mar_cpg_name_t (&req_lib_cpg_join.group_name,
 		group);
 
-	iov[0].iov_base = &req_lib_cpg_join;
+	iov[0].iov_base = (char *)&req_lib_cpg_join;
 	iov[0].iov_len = sizeof (struct req_lib_cpg_join);
 
 	error = saSendMsgReceiveReply (cpg_inst->response_fd, iov, 1,
@@ -513,7 +471,7 @@ cpg_error_t cpg_leave (
 	marshall_to_mar_cpg_name_t (&req_lib_cpg_leave.group_name,
 		group);
 
-	iov[0].iov_base = &req_lib_cpg_leave;
+	iov[0].iov_base = (char *)&req_lib_cpg_leave;
 	iov[0].iov_len = sizeof (struct req_lib_cpg_leave);
 
 	pthread_mutex_lock (&cpg_inst->response_mutex);
@@ -564,7 +522,7 @@ cpg_error_t cpg_mcast_joined (
 	req_lib_cpg_mcast.guarantee = guarantee;
 	req_lib_cpg_mcast.msglen = msg_len;
 
-	iov[0].iov_base = &req_lib_cpg_mcast;
+	iov[0].iov_base = (char *)&req_lib_cpg_mcast;
 	iov[0].iov_len = sizeof (struct req_lib_cpg_mcast);
 	memcpy (&iov[1], iovec, iov_len * sizeof (struct iovec));
 
@@ -580,6 +538,9 @@ cpg_error_t cpg_mcast_joined (
 	}
 
 	cpg_inst->flow_control_state = res_lib_cpg_mcast.flow_control_state;
+	if (res_lib_cpg_mcast.header.error == CPG_ERR_TRY_AGAIN) {
+		cpg_inst->flow_control_state = CPG_FLOW_CONTROL_ENABLED;
+	}
 	error = res_lib_cpg_mcast.header.error;
 
 error_exit:
@@ -611,7 +572,7 @@ cpg_error_t cpg_membership_get (
 	marshall_to_mar_cpg_name_t (&req_lib_cpg_membership_get.group_name,
 		group_name);
 
-	iov.iov_base = &req_lib_cpg_membership_get;
+	iov.iov_base = (char *)&req_lib_cpg_membership_get;
 	iov.iov_len = sizeof (mar_req_header_t);
 
 	pthread_mutex_lock (&cpg_inst->response_mutex);
@@ -644,48 +605,6 @@ error_exit:
 	return (error);
 }
 
-cpg_error_t cpg_local_get (
-	cpg_handle_t handle,
-	unsigned int *local_nodeid)
-{
-	cpg_error_t error;
-	struct cpg_inst *cpg_inst;
-	struct iovec iov;
-	struct req_lib_cpg_local_get req_lib_cpg_local_get;
-	struct res_lib_cpg_local_get res_lib_cpg_local_get;
-
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != SA_AIS_OK) {
-		return (error);
-	}
-
-	req_lib_cpg_local_get.header.size = sizeof (mar_req_header_t);
-	req_lib_cpg_local_get.header.id = MESSAGE_REQ_CPG_LOCAL_GET;
-
-	iov.iov_base = &req_lib_cpg_local_get;
-	iov.iov_len = sizeof (struct req_lib_cpg_local_get);
-
-	pthread_mutex_lock (&cpg_inst->response_mutex);
-
-	error = saSendMsgReceiveReply (cpg_inst->response_fd, &iov, 1,
-		&res_lib_cpg_local_get, sizeof (res_lib_cpg_local_get));
-
-	pthread_mutex_unlock (&cpg_inst->response_mutex);
-
-	if (error != SA_AIS_OK) {
-		goto error_exit;
-	}
-
-	error = res_lib_cpg_local_get.header.error;
-
-	*local_nodeid = res_lib_cpg_local_get.local_nodeid;
-
-error_exit:
-	saHandleInstancePut (&cpg_handle_t_db, handle);
-
-	return (error);
-}
-
 cpg_error_t cpg_flow_control_state_get (
 	cpg_handle_t handle,
 	cpg_flow_control_state_t *flow_control_state)
@@ -697,7 +616,7 @@ cpg_error_t cpg_flow_control_state_get (
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
-	
+
 	*flow_control_state = cpg_inst->flow_control_state;
 
 	saHandleInstancePut (&cpg_handle_t_db, handle);
