@@ -1,7 +1,6 @@
 /*
  * Copyright (c) 2004-2005 Mark Haverkamp
  * Copyright (c) 2004-2005 Open Source Development Lab
- * Copyright (c) 2006-2007 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -882,7 +881,7 @@ saEvtFinalize(SaEvtHandleT evtHandle)
 		return error;
 	}
 
-	pthread_mutex_lock(&evti->ei_response_mutex);
+       pthread_mutex_lock(&evti->ei_response_mutex);
 
 	/*
 	 * Another thread has already started finalizing
@@ -895,10 +894,7 @@ saEvtFinalize(SaEvtHandleT evtHandle)
 
 	evti->ei_finalize = 1;
 
-	pthread_mutex_unlock (&evti->ei_response_mutex);
- 	pthread_mutex_destroy (&evti->ei_response_mutex);
-
- 	pthread_mutex_destroy (&evti->ei_dispatch_mutex);
+	pthread_mutex_unlock(&evti->ei_response_mutex);
 
 	saHandleDestroy(&evt_instance_handle_db, evtHandle);
 	/*
@@ -994,7 +990,7 @@ saEvtChannelOpen(
 	req.ico_open_flag = channelOpenFlags;
 	marshall_to_mar_name_t (&req.ico_channel_name, (SaNameT *)channelName);
 
-	iov.iov_base = &req;
+	iov.iov_base = (char *)&req;
 	iov.iov_len = sizeof(req);
 
 	pthread_mutex_lock(&evti->ei_response_mutex);
@@ -1089,6 +1085,7 @@ saEvtChannelClose(SaEvtChannelHandleT channelHandle)
 	}
 	eci->eci_closing = 1;
 	pthread_mutex_unlock(&eci->eci_mutex);
+	
 
 	/*
 	 * Send the request to the server and wait for a response
@@ -1097,7 +1094,7 @@ saEvtChannelClose(SaEvtChannelHandleT channelHandle)
 	req.icc_head.id = MESSAGE_REQ_EVT_CLOSE_CHANNEL;
 	req.icc_channel_handle = eci->eci_svr_channel_handle;
 
-	iov.iov_base = &req;
+	iov.iov_base = (char *)&req;
 	iov.iov_len = sizeof (req);
 
 	pthread_mutex_lock(&evti->ei_response_mutex);
@@ -1125,8 +1122,6 @@ saEvtChannelClose(SaEvtChannelHandleT channelHandle)
 		goto chan_close_put2;
 	}
 
-	pthread_mutex_destroy(&eci->eci_mutex);
-    
 	saHandleInstancePut(&evt_instance_handle_db, 
 					eci->eci_instance_handle);
 	saHandleDestroy(&channel_handle_db, channelHandle);
@@ -1220,7 +1215,7 @@ saEvtChannelOpenAsync(SaEvtHandleT evtHandle,
 	req.ico_invocation = invocation;
 	req.ico_open_flag = channelOpenFlags;
 	marshall_to_mar_name_t (&req.ico_channel_name, (SaNameT *)channelName);
-	iov.iov_base = &req;
+	iov.iov_base = (char *)&req;
 	iov.iov_len = sizeof(req);
 
 
@@ -1321,7 +1316,7 @@ saEvtChannelUnlink(
 	req.iuc_head.size = sizeof(req);
 	req.iuc_head.id = MESSAGE_REQ_EVT_UNLINK_CHANNEL;
 	marshall_to_mar_name_t (&req.iuc_channel_name, (SaNameT *)channelName);
-	iov.iov_base = &req;
+	iov.iov_base = (char *)&req;
 	iov.iov_len = sizeof(req);
 
 
@@ -1460,7 +1455,6 @@ saEvtEventFree(SaEvtEventHandleT eventHandle)
 	edi->edi_freeing = 1;
 
 	pthread_mutex_unlock(&edi->edi_mutex);
-	pthread_mutex_destroy(&edi->edi_mutex);
 	saHandleDestroy(&event_handle_db, eventHandle);
 	saHandleInstancePut(&event_handle_db, eventHandle);
 
@@ -1998,7 +1992,7 @@ saEvtEventPublish(
 	req->led_priority = edi->edi_priority;
 	marshall_to_mar_name_t (&req->led_publisher_name, &edi->edi_pub_name);
 
-	iov.iov_base = req;
+	iov.iov_base = (char *)req;
 	iov.iov_len = req->led_head.size;
 
 	pthread_mutex_lock(&evti->ei_response_mutex);
@@ -2116,7 +2110,7 @@ saEvtEventSubscribe(
 	req->ics_channel_handle = eci->eci_svr_channel_handle;
 	req->ics_sub_id = subscriptionId;
 	req->ics_filter_size = sz;
-	iov.iov_base = req;
+	iov.iov_base = (char *)req;
 	iov.iov_len = req->ics_head.size;
 
 	pthread_mutex_lock(&evti->ei_response_mutex);
@@ -2179,7 +2173,7 @@ saEvtEventUnsubscribe(
 
 	req.icu_channel_handle = eci->eci_svr_channel_handle;
 	req.icu_sub_id = subscriptionId;
-	iov.iov_base = &req;
+	iov.iov_base = (char *)&req;
 	iov.iov_len = sizeof(req);
 
 	pthread_mutex_lock(&evti->ei_response_mutex);
@@ -2250,7 +2244,7 @@ saEvtEventRetentionTimeClear(
 	req.iec_channel_handle = eci->eci_svr_channel_handle;
 	req.iec_event_id = eventId;
 
-	iov.iov_base = &req;
+	iov.iov_base = (char *)&req;
 	iov.iov_len = sizeof(req);
 
 	pthread_mutex_lock(&evti->ei_response_mutex);
