@@ -1,10 +1,9 @@
 /*
  * Copyright (c) 2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * This software licensed under BSD license, the text of which follows:
  * 
@@ -588,7 +587,6 @@ saLckFinalize (
 	lckInstance->finalize = 1;
 
 	pthread_mutex_unlock (&lckInstance->response_mutex);
-	pthread_mutex_destroy (&lckInstance->response_mutex);
 
 // TODO	lckInstanceFinalize (lckInstance);
 
