@@ -1,11 +1,11 @@
 #define _BSD_SOURCE
 /*
  * Copyright (c) 2002-2004 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Sun Microsystems, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * This software licensed under BSD license, the text of which follows:
  * 
@@ -53,17 +53,21 @@
 
 #include "saAis.h"
 #include "saCkpt.h"
-#include "sa_error.h"
+
+#ifdef OPENAIS_SOLARIS
+#define timersub(a, b, result)						\
+    do {								\
+	(result)->tv_sec = (a)->tv_sec - (b)->tv_sec;			\
+	(result)->tv_usec = (a)->tv_usec - (b)->tv_usec;		\
+	if ((result)->tv_usec < 0) {					\
+	    --(result)->tv_sec;						\
+	    (result)->tv_usec += 1000000;				\
+	}								\
+    } while (0)
+#endif
 
 int alarm_notice;
 
-void fail_on_error(SaAisErrorT error, char* opName) {
-	if (error != SA_AIS_OK) {
-        printf ("%s: result %s\n", opName, get_sa_error_b(error));
-        exit (1);
-	}
-}
-
 void printSaNameT (SaNameT *name)
 {
 	int i;
@@ -88,7 +92,7 @@ SaCkptCheckpointCreationAttributesT checkpointCreationAttributes = {
         .retentionDuration =    0,
         .maxSections =          5,
         .maxSectionSize =       250000,
-        .maxSectionIdSize =     15
+        .maxSectionIdSize =     10
 };
 
 SaCkptSectionIdT sectionId1 = {
@@ -191,7 +195,10 @@ retry:
 		if (error == SA_AIS_ERR_TRY_AGAIN) {
 			goto retry;
 		}
-		fail_on_error(error, "saCkptCheckpointWrite");
+		if (error != SA_AIS_OK) {
+			printf ("saCkptCheckpointWrite result %d (should be 1)\n", error);
+			exit (1);
+		}
 		write_count += 1;
 	} while (alarm_notice == 0);
 	gettimeofday (&tv2, NULL);
@@ -222,25 +229,21 @@ int main (void) {
 	signal (SIGALRM, sigalrm_handler);
 
     error = saCkptInitialize (&ckptHandle, &callbacks, &version);
-	fail_on_error(error, "saCkptInitialize");
-	
+
 	error = saCkptCheckpointOpen (ckptHandle,
 		&checkpointName,
 		&checkpointCreationAttributes,
 		SA_CKPT_CHECKPOINT_CREATE|SA_CKPT_CHECKPOINT_READ|SA_CKPT_CHECKPOINT_WRITE,
 		0,
 		&checkpointHandle);
-	fail_on_error(error, "saCkptCheckpointOpen");
 	error = saCkptSectionCreate (checkpointHandle,
 		&sectionCreationAttributes1,
 		"Initial Data #0",
 		strlen ("Initial Data #0") + 1);
-	fail_on_error(error, "saCkptCheckpointSectionCreate");
 	error = saCkptSectionCreate (checkpointHandle,
 		&sectionCreationAttributes2,
 		"Initial Data #0",
 		strlen ("Initial Data #0") + 1);
-	fail_on_error(error, "saCkptCheckpointSectionCreate");
 
 	size = 1;
 
