@@ -1,11 +1,11 @@
 #define _BSD_SOURCE
 /*
  * Copyright (c) 2002-2004 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Sun Microsystems, Inc.
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@redhat.com)
+ * Author: Steven Dake (sdake@mvista.com)
  *
  * This software licensed under BSD license, the text of which follows:
  * 
@@ -51,17 +51,21 @@
 
 #include "saAis.h"
 #include "saCkpt.h"
-#include "sa_error.h"
+
+#ifdef OPENAIS_SOLARIS
+#define timersub(a, b, result)						\
+    do {								\
+	(result)->tv_sec = (a)->tv_sec - (b)->tv_sec;			\
+	(result)->tv_usec = (a)->tv_usec - (b)->tv_usec;		\
+	if ((result)->tv_usec < 0) {					\
+	    --(result)->tv_sec;						\
+	    (result)->tv_usec += 1000000;				\
+	}								\
+    } while (0)
+#endif
 
 int alarm_notice = 0;
 
-void fail_on_error(SaAisErrorT error, char* opName) {
-	if (error != SA_AIS_OK) {
-        printf ("%s: result %s\n", opName, get_sa_error_b(error));
-        exit (1);
-	}
-}
-
 void printSaNameT (SaNameT *name)
 {
 	int i;
@@ -84,7 +88,7 @@ SaCkptCheckpointCreationAttributesT checkpointCreationAttributes = {
 	.retentionDuration =	0,
 	.maxSections =		5,
 	.maxSectionSize =	150000,
-	.maxSectionIdSize =	15
+	.maxSectionIdSize =	10
 };
 
 SaCkptSectionIdT sectionId1 = {
@@ -174,6 +178,7 @@ struct threaddata {
 	int written;
 };
 
+extern void pthread_exit(void *) __attribute__((noreturn));
 
 void *benchmark_thread (void *arg) 
 {
@@ -201,7 +206,10 @@ void *benchmark_thread (void *arg)
 			1,
 			&erroroneousVectorIndex);
 		} while (error == SA_AIS_ERR_TRY_AGAIN);
-		fail_on_error(error, "saCkptCheckpointWrite");
+		if (error != SA_AIS_OK) {
+			printf ("saCkptCheckpointWrite result %d handle (should be 1)\n", error);
+			exit (1);
+		}
 		td->written += 1;
 	} while (alarm_notice == 0);
 	pthread_exit (0);
@@ -282,7 +290,7 @@ int main (void) {
 	 */
 	for (i  = 0; i < CHECKPOINT_THREADS_MAX; i++) {
 		sprintf ((char *)checkpointName.value, "checkpoint (%d)", i);
-		checkpointName.length = strlen (checkpointName.value);
+		checkpointName.length = strlen ((char *)checkpointName.value);
 		do {
 			error = saCkptInitialize (&ckpt_handles[i], &callbacks, &version);
 		} while (error == SA_AIS_ERR_TRY_AGAIN);
